import  os
import  datetime
import  threading
import  inspect
import  logging

# pfioh local dependencies
try:
    from    .message        import Message
except:
    from    message         import Message

logging.basicConfig(level=logging.DEBUG,
                    format='(%(threadName)-10s) %(message)s')

class debug(object):
    """
        A simple class that provides some helper debug functions. Mostly
        printing function/thread names and checking verbosity level
        before printing.
    """

    def log(self, *args):
        """
        get/set the log object.

        Caller can further manipulate the log object with object-specific
        calls.
        """
        if len(args):
            self._log = args[0]
        else:
            return self._log

    def name(self, *args):
        """
        get/set the descriptive name text of this object.
        """
        if len(args):
            self.__name = args[0]
        else:
            return self.__name

    def __init__(self, **kwargs):
        """
        Constructor
        """

        self.verbosity  = 0
        self.level      = 0

        self.b_useDebug             = False
        self.str_debugDirFile       = '/tmp'
        for k, v in kwargs.items():
            if k == 'verbosity':    self.verbosity          = v
            if k == 'level':        self.level              = v
            if k == 'debugToFile':  self.b_useDebug         = v
            if k == 'debugFile':    self.str_debugDirFile   = v

        if self.b_useDebug:
            str_debugDir                = os.path.dirname(self.str_debugDirFile)
            str_debugName               = os.path.basename(self.str_debugDirFile)
            if not os.path.exists(str_debugDir):
                os.makedirs(str_debugDir)
            self.str_debugFile          = '%s/%s' % (str_debugDir, str_debugName)
            self.debug                  = Message(logTo = self.str_debugFile)
            self.debug._b_syslog        = False
            self.debug._b_flushNewLine  = True
        self._log                   = Message()
        self._log._b_syslog         = True
        self.__name                 = "pman"


    def __call__(self, *args, **kwargs):
        self.qprint(*args, **kwargs)

    def qprint(self, *args, **kwargs):
        """
        The "print" command for this object.

        :param kwargs:
        :return:
        """

        self.level  = 0
        self.msg    = ""

        for k, v in kwargs.items():
            if k == 'level':    self.level  = v
            if k == 'msg':      self.msg    = v

        if len(args):
            self.msg    = args[0]

        if self.b_useDebug:
            write   = self.debug
        else:
            write   = print

        if self.level <= self.verbosity:

            if self.b_useDebug:
                write('| %50s | %30s | ' % (
                    threading.current_thread(),
                    inspect.stack()[1][3]
                ), end='', syslog = True)
            else:
                write('%26s | %50s | %30s | ' % (
                    datetime.datetime.now(),
                    threading.current_thread(),
                    inspect.stack()[1][3]
                ), end='')
            for t in range(0, self.level): write("\t", end='')
            write(self.msg)


#!/usr/bin/env python
"""
    NAME

        C_stringCore

    DESCRIPTION
        
        'C_stringCore' is a wrapper about a python cStringIO
        class. It provides a thins wrapper abstracting some
        simple methods to write/read to and from a StringIO 
        object.

    NOTES

    HISTORY

        06 February 2008
        o Design consolidated from several sources.

        25 February 2014
        o print --> print()

        27 February 2014
        o Fix class/instance variable mixup
        o Refactor and cleanup

"""

# System modules
import  os
import  sys
from    string          import  *

# doesn't seem right?
try:
	from    cStringIO       import  StringIO
except ImportError:
	from 	io 		import 	StringIO
from    cgi             import  *

class C_stringCore:
        """
        This class is a wrapper about a cStringIO instance, keeping
        track of an internal file-string instance and syncing its
        contents to an internal string buffer.
        """

        
        #
        # Methods
        #
        def __init__(self):
                # 
                # Member variables
                #
                #       - Core variables
                self.str_obj    = 'C_stringCore'        # name of object class
                self.str_name   = 'void'                # name of object variable
                self._id        = -1                    # id of agent
                self._iter      = 0                     # current iteration in an
                                                        #       arbitrary processing 
                                                        #       scheme
                self._verbosity = 0                     # debug related value for
                                                        #       object
                self._warnings  = 0                     # show warnings
                                                        #       (and warnings level)
                
                #
                #       - Class variables
                self.StringIO           = StringIO()    # A file string buffer that
                                                        #       functions as a 
                                                        #       scratch space for
                                                        #       the core
                self.str                = ""

        def metaData_print(self):
                print('str_obj\t\t= %s'         % self.str_obj)
                print('str_name\t\t= %s'        % self.str_name)
                print('_id\t\t\t= %d'           % self._id)
                print('_iter\t\t\t= %d'         % self._iter)
                print('_verbosity\t\t= %d'      % self._verbosity)
                print('_warnings\t\t= %d'       % self._warnings)
                return 'This class functions as a string file handler.'

        def __str__(self):
                return self.str 
        
        #
        # core methods
        def strout(self, astr_text=""):
            if(len(astr_text)): 
                self.write(astr_text)
                print("%s" % self.strget())

        def reset(self, astr_newCore = ""):
            self.StringIO.close()
            self.StringIO       = StringIO()
            self.StringIO.write(astr_newCore)

        def strget(self):
            return self.StringIO.getvalue()

        def write(self, astr_text):
            if isinstance(astr_text, list):
              astr_text         = '\n'.join(astr_text)
            self.StringIO.write(astr_text)
            self.str            = self.strget()
            return astr_text



#!/usr/bin/env python3
# 
# NAME
#
#	dgmsocket class
#
# DESCRIPTION
#
#	The 'dgmsocket' class provides a very simple wrapper the standard
#	python socket API.
#
#	More specifically, this class provides datagram socket services.
#
# HISTORY
#
# 25 March 2006
# o Initial development implementation
#
# 06 December 2011
# o Clean-up the socket communication
#
import socket

class C_dgmsocket :
        #
        # Member variables
        #
        # 	- Core variables
        mstr_obj	= 'C_dgmsocket' # name of object class
        mstr_name	= 'void'        # name of object variable
        m_id		= -1            # id of agent
        m_iter		= 0             # current iteration in an
                                        #  	arbitrary processing
                                        #	scheme
        m_verbosity	= 0		# debug related value for
                                        #	object
        m_warnings	= 0             # show warnings
                                        #	(and warnings level)

        #
        #	- Class variables
        m_dgmsocket	    = None
        mstr_remoteHost	    = 'localhost'
        m_port		    = 1701

        #
        # Methods
        #
        # Core methods - construct, initialise, id
        def core_construct(	self,
                                astr_obj	= 'C_dgmsocket',
                                astr_name	= 'void',
                                a_id		= -1,
                                a_iter		= 0,
                                a_verbosity	= 0,
                                a_warnings	= 0) :
                self.mstr_obj		= astr_obj
                self.mstr_name		= astr_name
                self.m_id		= a_id
                self.m_iter		= a_iter
                self.m_verbosity	= a_verbosity
                self.m_warnings		= a_warnings

        def __str__(self):
                print('mstr_obj\t\t= %s' 	% self.mstr_obj)
                print('mstr_name\t\t= %s' 	% self.mstr_name)
                print('m_id\t\t\t= %d' 		% self.m_id)
                print('m_iter\t\t\t= %d'	% self.m_iter)
                print('m_verbosity\t\t= %d'	% self.m_verbosity)
                print('m_warnings\t\t= %d'	% self.m_warnings)
                return 'This class provides a *very* simple wrapper framework about datagram sockets.'

        def __init__(self, astr_hostname = 'localhost', a_port = 1701):
                self.core_construct()
                self.mstr_remoteHost    = astr_hostname
                self.m_port 		= a_port
                self.m_dgmsocket        = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)

        def tx(self, str_payload):
                self.m_dgmsocket.sendto(str_payload.encode(), (self.mstr_remoteHost, self.m_port))

        def write(self, str_payload):
                self.m_dgmsocket.sendto(str_payload.encode(), (self.mstr_remoteHost, self.m_port))

        def close(self):
                self.m_dgmsocket.close()

        def flush(self):
                pass
            
                

